require "toni/role"
require "toni/permission"
require "rspec/matchers"

module Toni
  class Builder

    attr_reader :roles

    def initialize(authorization_rules)
      @roles = {}
      instance_eval authorization_rules, Toni::AUTH_FILE
    end

    def role(role_symbol, &block)
      builder = RoleBuilder.new(role_symbol, &block)
      @roles[role_symbol] = builder.role
    end

    class RoleBuilder

      attr_reader :role

      def initialize(role_symbol, &block)
        @role = Role.new(role_symbol)
        if block_given?
          instance_eval &block
        end
      end

      # Add an ancestor to the current role
      def includes(role_symbol)
        @role.add_ancestor(role_symbol)
      end

      # Add permissions to specific role
      def has_permission_on(resource_name, options={}, &block)
        builder = PermissionBuilder.new(resource_name, options, &block)
        @role.add_permission(builder.permission)
      end

    end

    class PermissionBuilder

      attr_reader :permission

      def initialize(resource_name, options={}, &block)
        @permission = Permission.new(resource_name, [*options[:to]])
        if block_given?
          instance_eval &block
        end
      end

      # Attribute should always be a column in the database on the resource
      def expect_attribute(method_name, &block)
        expect_method(method_name, true, &block)
      end

      # Methods should be defined on the resource
      def expect_method(method_name, is_attr = false, &block)
        builder = ExpectationBuilder.new(&block)
        permission.add_matcher(method_name, builder, is_attr)
      end

    end

    class ExpectationBuilder

      include ::RSpec::Matchers

      def initialize(&block)
        @block = block
      end

      def execute
        unless @block.nil?
          instance_eval &@block
        end
      end

      def current_user
        Toni.current_user
      end

      def not
        Toni::PermissionMatcher::NotMatcher.new
      end

      def permitted_to(activity)
        Toni::PermissionMatcher::PermittedToMatcher.new(activity)
      end

    end

  end
end
